#include <iostream>
#include <string>
#include <stdexcept>
#include <map>

#include "include/tcon_dev.h"

int main(int argc, char *argv[]) {
    std::map<short, std::string> error_to_string = {
            { LIBTCON_SUCCESS, "Success" },
            { LIBTCON_ERROR_IO, "Input / Output error" },
            { LIBTCON_ERROR_INVALID_PARAM, "Invalid parameters" },
            { LIBTCON_ERROR_ACCESS, "Access denied (insufficient permission)" },
            { LIBTCON_ERROR_NO_DEVICE, "No such device" },
            { LIBTCON_ERROR_NOT_FOUND, "Entity not found" },
            { LIBTCON_ERROR_BUSY, "Resource busy" },
            { LIBTCON_ERROR_TIMEOUT, "Operation timed out" },
            { LIBTCON_ERROR_OVERFLOW, "Overflow" },
            { LIBTCON_ERROR_PIPE, "Pipe error" },
            { LIBTCON_ERROR_INTERRUPTED, "System call interrupted" },
            { LIBTCON_ERROR_NO_MEM, "Insufficient memory" },
            { LIBTCON_ERROR_NOT_SUPPORTED, "Operation not supported or unimplemented on this platform" },
            { LIBTCON_ERROR_OTHER, "Other error" },
        };

    std::cout << "ejump_waveform - Sets waveform for connected eJump T-CON board" << std::endl;
    std::cout << std::endl;

    if(argc != 2) {
        std::cout << "Usage (assuming waveform file waveform.wbf in same folder):" << std::endl;
        std::cout << "sudo ./ejump_vcom ./waveform.wbf" << std::endl;
        return 1;
    }

    std::cout << "Trying to initialize T-CON board..." << std::endl;
    short board_count = tcon_init();
    if(board_count < 0) {
        std::cerr << "Error: Failed to initialize T-CON board for reason: " << error_to_string[board_count] << std::endl;
        return board_count;
    } else if(board_count == 0) {
        std::cerr << "Error: No T-CON boards detected!" << std::endl;
        return LIBTCON_ERROR_NO_DEVICE;
    } else {
        std::cout << "Successfully initialized T-CON board!" << std::endl;
    }

    // -1: Target T-CON board will be first board to be found via USB
    short board_id = -1;

    if(short retval = tcon_upd_wf(board_id, argv[1]); retval != LIBTCON_SUCCESS) {
        std::cerr << "Error: Failed to update waveform for reason: " << error_to_string[retval] << std::endl;
        return LIBTCON_ERROR_OTHER;
    } else {
        std::cout << "Successfully updated waveform!" << std::endl;
        return 0;
    }
}
